# type: ignore
"""A test module.

Contains some functions with no docstrings.
"""


def clean_trailing_spaces() -> None:  # noqa: D103
    pass


def add_indentation_spaces() -> None:  # noqa: D103
    pass


def remove_end_of_line() -> None:  # noqa: D103
    pass


def add_end_of_line() -> None:  # noqa: D103
    pass
