# Copyright (c) 2024 Valentin Goldite. All Rights Reserved.
"""Docstripy - Convert any python docstrings with the docstring format you want.

Support Nympydoc, Google and ReStructuredText as output styles.
The input style should be either Numpy, Google, ReST or even a mix of both.
"""
