dd p.rubric,

dl[class]:not(.option-list):not(.field-list):not(.footnote):not(.glossary):not(.simple) .field-list>dt {
    text-transform: capitalize;
    font-weight: bold;
    font-size: 115%;
    background-clip: padding-box;
}

.sidebar-brand-text {
    --color-sidebar-brand-text: #27e560;
}

.sidebar-tree {
    font-size: 105%;
    --color-sidebar-link-text--top-level: #49dd9b;
    --color-sidebar-link-text: #9e9e9e;
}

.sig-name {
    color: #49dd9b;
}

html {
    font-size: 100%;
}

.sig:not(.sig-inline) {
    font-family: var(--font-stack--monospace);
    font-size: 105%;
    font-weight: 700;
    text-indent: -2.5em;
}

.sig .n {
    color: #2b6ea8;
}

body:not([data-theme="light"]) {
    --color-highlight-on-target: #222222;
    /* default #131416 */
    --color-background-primary: #1d1d1d;
    /* default #1a1c1e */
    --color-background-secondary: #222222;
}